#!/usr/bin/python3

import urllib.request
import sys, ssl, json, pprint

context = ssl._create_unverified_context()

# Hack: Set verbose if -v parameter given, remove that from sys.argv
verbose = '-v' in sys.argv
if '-v' in sys.argv: sys.argv.remove('-v')

def put(url, content):
    if verbose: print(url, content)
    req = urllib.request.Request(url=url, data=content.encode('UTF-8'),
            method='PUT')
    f = urllib.request.urlopen(req, context=context)
    if verbose: print(f.status, f.reason)

def get_json(url):
    if verbose: print(url)
    req = urllib.request.Request(url=url, method='GET')
    f = urllib.request.urlopen(req, context=context)
    if verbose: print(f.status, f.reason)
    return json.loads(f.read())

def rgb2xyb(r,g,b):
    r = ((r+0.055)/1.055)**2.4 if r > 0.04045 else r/12.92
    g = ((g+0.055)/1.055)**2.4 if g > 0.04045 else g/12.92
    b = ((b+0.055)/1.055)**2.4 if b > 0.04045 else b/12.92

    X = r * 0.4124 + g * 0.3576 + b * 0.1805
    Y = r * 0.2126 + g * 0.7152 + b * 0.0722
    Z = r * 0.0193 + g * 0.1192 + b * 0.9505

    return X / (X + Y + Z), Y / (X + Y + Z), int(Y*254)

# Change these to match your setup
user = 'yourusername'
ip = '192.168.1.123'

if len(sys.argv) < 2 or sys.argv[1] in ('-h', '--help', 'help'):
    print('Usage: python3 hue.py ...')
    print('  list              List lights')
    print('  on N ...          Turn light(s) N ... on')
    print('  rgb R G B N ...   Set light(s) N ... to R, G, B values (float)')
    print('  ctb CT B N ...    Set light(s) N ... to color temp (154-500) and brightness (0-254)')
    print('  xyb X Y B N ...   Set light(s) N ... to X,Y (float) and brightness (0-254)')
    print('  off N ...         Turn light(s) N ... off')
    print('  light N ...       Get light N data')
    print('  state N ...       Get light(s) N ... state')
    print('  getxyb N ...      Get light(s) N ... settings in X Y B format')
    print('  setxyb file       Set lights specified in file using format from getxyb')
    exit(0)

pp = pprint.PrettyPrinter(indent=4)   

if sys.argv[1] == 'list':
    data = get_json(f'https://{ip}/api/{user}/lights')
    for k in data:
        print(k, data[k]['name'])
elif sys.argv[1] in ('on', 'off'):
    content = '{"on":true}' if sys.argv[1]=='on' else '{"on":false}'
    for light in sys.argv[2:]:
        put(f'https://{ip}/api/{user}/lights/{light}/state', content)
elif sys.argv[1] == 'rgb':
    x,y,b = rgb2xyb(*(float(v) for v in sys.argv[2:5]))
    state = '{"on":true,"bri":%d,"xy":[%.4f,%.4f]}' % (b, x, y)
    for light in sys.argv[5:]:
        put(f'https://{ip}/api/{user}/lights/{light}/state', state)
elif sys.argv[1] == 'ctb':
    ct,b = (int(v) for v in sys.argv[2:4])
    state = '{"on":true,"ct":%d,"bri":%d}' % (ct,b)
    for light in sys.argv[4:]:
        put(f'https://{ip}/api/{user}/lights/{light}/state', state)
elif sys.argv[1] == 'xyb':
    x,y,b = float(sys.argv[2]), float(sys.argv[3]), int(sys.argv[4])
    state = '{"on":true,"xy":[%.4f,%.4f],"bri":%d}' % (x,y,b)
    for light in sys.argv[4:]:
        put(f'https://{ip}/api/{user}/lights/{light}/state', state)
elif sys.argv[1] in ('light', 'state', 'getxyb'):
    for light in sys.argv[2:]:
        state = get_json(f'https://{ip}/api/{user}/lights/{light}')
        if sys.argv[1] != 'light': state = state['state']
        if sys.argv[1] == 'getxyb': print(state['xy'][0], state['xy'][1], state['bri'], light)
        else: pp.pprint(state)
elif sys.argv[1] == 'setxyb':
    with open(sys.argv[2]) as fin:
        for line in fin:
            x,y,b,light = line.split()
            state = '{"on":true,"xy":[%s,%s],"bri":%s}' % (x,y,b)
            put(f'https://{ip}/api/{user}/lights/{light}/state', state)